from peptides import Peptide


class Pep:
    def __init__(self, seq: str) -> None:
        self.peptide = Peptide(seq)

    @property
    def vhse_scales(self):
        return self.peptide.vhse_scales()

    @property
    def z_scales(self):
        return self.peptide.z_scales()



from torchtext.vocab import vocab
from collections import OrderedDict
import torch
from pathlib import Path

import io, os


class Vocab:
    def __init__(self, file_path=None) -> None:
        if file_path:
            self.file_path = file_path
        else:
            self.file_path = (
                Path(os.path.dirname(os.path.dirname(__file__))) / "resource/vocab.txt"
            )
        self.unk_token = "<unk>"
        self.default_index = -1
        self.v = self.get_vocab()

    def yield_tokens(self):
        with io.open(self.file_path, encoding="utf-8") as f:
            for line in f:
                yield line.strip()

    @property
    def tokens(self):
        return self.yield_tokens()

    def get_vocab(self):
        v = vocab(
            OrderedDict([(token, 1) for token in self.tokens]),
            specials=[self.unk_token],
        )
        v.set_default_index(self.default_index)
        return v

    def ids(self, seq):
        return [self.v[i] for i in seq]



from sklearn.metrics import (
    accuracy_score,
    confusion_matrix,
    classification_report,
    roc_curve,
    roc_auc_score,
)
import pandas as pd
import numpy as np
import seaborn as sns
import matplotlib.pyplot as plt


def show_classification(y_test, y_pred):
    r"""
    Confusion matrix

    - Binary:
      - y_test: [1, 0, 1, 0, 1]
      - y_pred: [0.1, 0.2, 0.9, 0, 0.8]
    - Multi:
      - y_test: [1, 2, 1, 0, 1]
      - y_pred: [[0.1, 0.8, 0.1], [0.1, 0.2, 0.7], [0.1, 0.6, 0.3], [0.5, 0.3. 0.2], [0.1, 0.6, 0.4]]
    """
    cm = confusion_matrix(y_test, y_pred)
    TN = cm[0, 0]
    TP = cm[1, 1]
    FP = cm[0, 1]
    FN = cm[1, 0]
    print(sum(y_test), sum(y_pred))
    print("Confusion matrix\n\n", cm)
    print("\nTrue Negatives(TN) = ", TN)
    print("\nTrue Positives(TP) = ", TP)
    print("\nFalse Positives(FP) = ", FP)
    print("\nFalse Negatives(FN) = ", FN)

    classification_accuracy = (TP + TN) / float(TP + TN + FP + FN)
    print("Classification accuracy : {0:0.4f}".format(classification_accuracy))
    classification_error = (FP + FN) / float(TP + TN + FP + FN)
    print("Classification error : {0:0.4f}".format(classification_error))
    precision = TP / float(TP + FP)
    print("Precision : {0:0.4f}".format(precision))
    recall = TP / float(TP + FN)
    print("Recall or Sensitivity : {0:0.4f}".format(recall))
    true_positive_rate = TP / float(TP + FN)
    print("True Positive Rate : {0:0.4f}".format(true_positive_rate))
    false_positive_rate = FP / float(FP + TN)
    print("False Positive Rate : {0:0.4f}".format(false_positive_rate))
    specificity = TN / (TN + FP)
    print("Specificity : {0:0.4f}".format(specificity))

    cm_matrix = pd.DataFrame(
        data=cm.T,
        columns=["Actual Negative:0", "Actual Positive:1"],
        index=["Predict Negative:0", "Predict Positive:1"],
    )
    sns.heatmap(cm_matrix, annot=True, fmt="d", cmap="YlGnBu")

    fpr, tpr, thresholds = roc_curve(y_test, y_pred)
    plt.figure(figsize=(6, 4))
    plt.plot(fpr, tpr, linewidth=2)
    plt.plot([0, 1], [0, 1], "k--")
    plt.rcParams["font.size"] = 12
    plt.title("ROC curve for Predicting a Pulsar Star classifier")
    plt.xlabel("False Positive Rate (1 - Specificity)")
    plt.ylabel("True Positive Rate (Sensitivity)")
    plt.show()

    ROC_AUC = roc_auc_score(y_test, y_pred)
    print("ROC AUC : {:.4f}".format(ROC_AUC))



import pandas as pd
from sklearn.utils import shuffle
from sklearn.model_selection import (
    StratifiedShuffleSplit,
    StratifiedKFold,
    KFold,
    train_test_split,
)
from typing import Optional, Union, List, Tuple
from pathlib import Path
import copy


class DataFrame():
    def __init__(
        self, df: pd.DataFrame, random_state: int = 42, *args, **kargs
    ) -> None:
        super(DataFrame, self).__init__()
        self.df = df.copy(deep=True)  # get a copy of original dataframe
        self.random_state = random_state

    def __repr__(self) -> str:
        return repr(self.df)

    def split_df(
        self,
        val: bool = False,
        test_size: float = 0.1,
        val_size: Optional[float] = None,
    ):
        if val:
            assert val_size is not None, "val size needed"
            val_num, test_num = len(self.df) * [val_size, test_size]

            train_df, val_df = train_test_split(
                self.df, test_size=int(val_num), random_state=self.random_state
            )
            train_df, test_df = train_test_split(
                train_df, test_size=int(test_num), random_state=self.random_state
            )
            return train_df, val_df, test_df
        else:
            train_df, test_df = train_test_split(
                train_df, test_size=test_size, random_state=self.random_state
            )
            return train_df, test_df

    def stratified_split_df(
        self, labels: Union[List[str], str], n_splits: int = 1, test_size: float = 0.1
    ) -> Union[List, Tuple]:
        split = StratifiedShuffleSplit(
            n_splits=n_splits, test_size=test_size, random_state=self.random_state
        )
        df_trains = []
        df_tests = []
        for train_index, test_index in split.split(self.df, self.df[labels]):
            strat_train_set = self.df.loc[train_index]
            strat_test_set = self.df.loc[test_index]
            df_trains.append(strat_train_set)
            df_tests.append(strat_test_set)

        return (
            (strat_train_set, strat_test_set)
            if n_splits == 1
            else (df_trains, df_tests)
        )

    def stratified_kfold_split_df(
        self, labels: Union[List[str], str], n_splits: int = 2
    ) -> Tuple:
        assert n_splits >= 2, "At least 2 fold"
        skf = StratifiedKFold(
            n_splits=n_splits, shuffle=True, random_state=self.random_state
        )
        for train_index, test_index in skf.split(self.df, self.df[labels]):
            strat_train_set = self.df.loc[train_index]
            strat_test_set = self.df.loc[test_index]
            yield strat_train_set, strat_test_set

    def kfold_split_df(self, labels: Union[List[str], str], n_splits: int = 2) -> Tuple:
        assert n_splits >= 2, "At least 2 fold"
        df_trains = []
        df_tests = []
        skf = StratifiedKFold(
            n_splits=n_splits, shuffle=True, random_state=self.random_state
        )
        for train_index, test_index in skf.split(self.df, self.df[labels]):
            strat_train_set = self.df.loc[train_index]
            strat_test_set = self.df.loc[test_index]
            df_trains.append(strat_train_set)
            df_tests.append(strat_test_set)

        return df_trains, df_tests

    def show_ratio(self, label="label", sort=None, n: Optional[int] = None):
        """print the label proportion in pd.DataFrame
        Args:
            sort: 'value' or 'label'
        """
        n_all = len(self.df)
        if sort == "value":
            n_classes = (
                self.df[label]
                .value_counts()
                .reset_index()
                .sort_values(by=label, ascending=False)
            )
        elif sort == "label":
            n_classes = (
                self.df[label].value_counts().reset_index().sort_values(by="index")
            )
        else:
            n_classes = self.df[label].value_counts().reset_index()
        if n:
            n_classes = n_classes[:n]

        for i in n_classes.index:
            print(
                f'Label, {n_classes.at[i, "index"]} takes: {n_classes.at[i, label] / n_all * 100:.2f}%, Nums: {n_classes.at[i, label]}'
            )


def read_csv(path: str = "", random_state: int = 42, *args, **kargs):
    _path = Path(path)
    assert _path.is_file(), "not a file"
    return DataFrame(df=pd.read_csv(path, *args, **kargs), random_state=random_state)


def split_df(df: pd.DataFrame, shuf=True, val=True, random_state=42):
    """Split df into train/val/test set and write into files
    ratio: 8:1:1 or 9:1

    Args:
        - df (DataFrame): some data
        - shuf (bool, default=True): shuffle the DataFrame
        - val (bool, default=True): split into three set, train/val/test
    """
    if shuf:
        df = shuffle(df, random_state=random_state)

    sep = int(len(df) * 0.1)

    if val:
        test_df = df.iloc[:sep]
        val_df = df.iloc[sep : sep * 2]
        train_df = df.iloc[sep * 2 :]
        return train_df, val_df, test_df
    else:
        test_df = df.iloc[:sep]
        train_df = df.iloc[sep:]
        return train_df, test_df


def show_ratio(df: pd.DataFrame, label="label", sort=None, n=5) -> None:
    """print the label proportion in pd.DataFrame
    Args:
        sort: 'value' or 'label'
    """
    n_all = len(df)
    if sort == "value":
        n_classes = (
            df[label]
            .value_counts()
            .reset_index()
            .sort_values(by=label, ascending=False)
        )
    elif sort == "label":
        n_classes = df[label].value_counts().reset_index().sort_values(by="index")
    else:
        n_classes = df[label].value_counts().reset_index()

    n_classes = n_classes[:n]

    for i in n_classes.index:
        print(
            f'Label {n_classes.at[i, "index"]} takes: {n_classes.at[i, label] / n_all * 100:.2f}%, Nums: {n_classes.at[i, label]}'
        )



from abc import ABC, abstractmethod
from typing import Optional, Iterable
from rdkit import Chem, RDLogger

RDLogger.DisableLog("rdApp.*")


def canonize_smi(smi: str):
    try:
        return Chem.MolToSmiles(Chem.MolFromSmiles(smi), canonical=True)
    except:
        return None


def canonize_rxn(rxn: str) -> Optional[str]:
    """标准化反应"""
    try:
        new_rxn = rxn.split(">>")
        if len(new_rxn) != 2:
            return None
        r = new_rxn[0]
        p = new_rxn[1]
        new_r = ".".join([canonize_smi(x) for x in r.split(".")])
        new_p = canonize_smi(p)
        return new_r + ">>" + new_p
    except:
        return None


def canonize_rxns(rxns: Iterable[str]):
    """批量标准化反应"""
    return [canonize_rxn(rxn) for rxn in rxns]


class SMI(object):
    def __init__(self, smi_str):
        self.smi_str = smi_str

    def __str__(self):
        return repr(self)

    def __repr__(self):
        return self.smi_str


class RXN(object):
    def __init__(
        self, rxn_str: str, yield_: Optional[float] = None, canonize: bool = True
    ):
        if canonize:
            self.rxn_str = canonize_rxn(rxn_str)
        else:
            self.rxn_str = rxn_str
        if yield_:
            self.yield_ = yield_

    @property
    def products(self):
        return self.rxn_str.split(">>")[1].split(".")

    @property
    def reactants(self):
        return self.rxn_str.split(">>")[0].split(".")

    def __str__(self):
        return repr(self)

    def __repr__(self):
        return self.rxn_str



from pathlib import Path
import shutil
from typing import Optional, Union, List

try:
    import gzip
    import tarfile
except:
    raise ImportError


def get_path(path: Union[Path, str]) -> Path:
    """Transform to `Path`.

    Args:
        path (str): The path to be transformed.

    Returns:
        Path: the `pathlib.Path` class
    """
    if isinstance(path, Path):
        return path
    else:
        return Path(path)


def get_path_out(
    path_in: Path, rename: str, path_out: Optional[Union[Path, str]] = None
):
    r"""
    Adaptor pathout to path_in
    """
    if path_out is None:
        return path_in.parent / rename
    else:
        _path_out = get_path(path_out)
        if _path_out.is_dir():
            return _path_out / rename
        elif path_out.is_file():
            return _path_out


def zip(path_in: Union[Path, str], path_out: Optional[Union[Path, str]] = None):
    r""" """
    _path_in = get_path(path_in)
    assert _path_in.is_file(), f"{path_in} is not a file"
    rename = _path_in.name + ".gz"
    _path_out = get_path_out(_path_in, rename, path_out)
    with open(_path_in, "rb") as f_in:
        with gzip.open(_path_out, "wb") as f_out:
            f_out.write(f_in.read())


def unzip(path_in: Union[Path, str], path_out: Optional[Union[Path, str]] = None):
    _path_in = get_path(path_in)
    assert _path_in.is_file(), f"{path_in} is not a file"
    assert _path_in.suffix == ".gz", f"not .gz file name"
    rename = _path_in.name.rstrip(".gz")  # rip
    _path_out = get_path_out(_path_in, rename, path_out)

    with gzip.open(_path_in, "rb") as f_in:
        with open(_path_out, "wb") as f_out:
            f_out.write(f_in.read())


def tar(
    path: Union[Path, str], staffs: Union[List[Union[Path, str]], Union[Path, str]]
):
    _path = get_path(path)
    with tarfile.open(_path, "w:gz") as tar:
        if isinstance(staffs, (str, Path)):
            tar.add(staffs)
            print(f"add {staffs}")
        elif isinstance(staffs, List):
            for staff in staffs:
                tar.add(staff)
                print(f"add {staff}")


def untar(path_in: Union[Path, str], path_out: Optional[Union[Path, str]] = None):
    _path_in = get_path(path_in)

    assert _path_in.is_file(), f"{path_in} is not a file"
    rename = _path_in.name.rstrip(".tar.gz")  # rip
    _path_out = get_path_out(_path_in, rename, path_out)

    with tarfile.open(_path_in, "r:gz") as tar:
        tar.extract(_path_out)



from z.text import Vocab
import unittest

class Test_Vocab(unittest.TestCase):

    def test_vocab(self):
        v = Vocab()
        vocab = v.get_vocab()
        print(v.file_path)
        print(list(v.tokens))
        for i in v.tokens:
            print(f'{i}: {vocab[i]}')
        print(f'<unk>: {vocab["<unk>"]}')
        print(f'outofdict: {vocab["outofdict"]}')

if __name__ == '__main__':
    unittest.main()


from rdkit import Chem, DataStructs


def smi2fp(smi):
    try:
        m = Chem.MolFromSmiles(smi)
        fp = Chem.RDKFingerprint(m)
        return fp
    except Exception as e:
        print(e)
        return None


def get_similarity(fp1, fp2):
    return DataStructs.FingerprintSimilarity(fp1, fp2)
